@@ -8,19 +8,34 @@ from pyrsistent import pmap, pvector
 from pyrsistent.typing import PMap
 
 from .errors import Invalid
-from .. import sums, flags
-from ..definitions import OverridesT
+from .. import sums
 from .. import interface as iface
-from ..compat import PY_VERSION
 from . import primitives
 from . import meta
 from . import nodes
 
 
-SchemaType = col.SchemaType
 Null = nodes.Null
 
 
+class TypedMapping(col.Mapping):
+    def __init__(self, *, key_node: nodes.SchemaNode, value_node: nodes.SchemaNode):
+        # https://docs.pylonsproject.org/projects/colander/en/latest/api.html#colander.Mapping
+        super().__init__(unknown='preserve')
+        self.key_node = key_node
+        self.value_node = value_node
+
+    def deserialize(self, node, cstruct):
+        r = super().deserialize(node, cstruct)
+        if r in (Null, None):
+            return r
+        return {self.key_node.deserialize(k): self.value_node.deserialize(v) for k, v in r.items()}
+
+    def serialize(self, node, appstruct):
+        r = super().serialize(node, appstruct)
+        return r
+
+
 class Path(primitives.Str):
     def __init__(self, typ: t.Type[pathlib.PurePath], *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
@@ -42,7 +57,7 @@ class Path(primitives.Str):
             raise Invalid(node, f'Invalid variant of {self.typ.__name__}', cstruct)
 
 
-class Structure(col.Mapping):
+class Structure(col.Mapping, metaclass=meta.SubscriptableSchemaTypeM):
     """ SchemaNode for NamedTuples and derived types.
     """
     def __init__(self,
@@ -75,7 +90,7 @@ class Structure(col.Mapping):
             for k, v in r.items()
         })
 
-    def serialize(self, node, appstruct: iface.IType):
+    def serialize(self, node, appstruct: iface.IType) -> t.Mapping[str, t.Any]:
         if appstruct is Null:
             return super().serialize(node, appstruct)
         return super().serialize(
@@ -91,7 +106,7 @@ class Tuple(col.Tuple, metaclass=meta.SubscriptableSchemaTypeM):
     pass
 
 
-class Sum(SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
+class Sum(meta.SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
     def __init__(
         self,
         typ: sums.SumType,
@@ -106,7 +121,7 @@ class Sum(SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
         self.typ = typ
         self.variant_nodes = variant_nodes
         self.as_dict_key = as_dict_key
-        self.variant_schema_types: t.Set[col.SchemaType] = {
+        self.variant_schema_types: t.Set[meta.SchemaType] = {
             x.typ for _, x in variant_nodes
         }
 
@@ -209,16 +224,10 @@ class Enum(primitives.Str):
             raise Invalid(node, f'Invalid variant of {self.typ.__name__}', cstruct)
 
 
+generic_type_bases: t.Callable[[t.Type], t.Tuple[t.Type, ...]] = lambda x: (insp.get_origin(x),)
 
 
-
-generic_type_bases: t.Callable[[t.Type], t.Tuple[t.Type, ...]] = (
-    insp.get_generic_bases if PY_VERSION < (3, 7) else
-    lambda x: (insp.get_origin(x),)
-)
-
-
-class Literal(SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
+class Literal(meta.SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
     def __init__(self, variants: t.FrozenSet):
         super().__init__()
         self.variants = variants
@@ -248,7 +257,7 @@ class Literal(SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
         )
 
 
-class Union(SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
+class Union(meta.SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
     """ This node handles typing.Union[T1, T2, ...] cases.
     Please note that typing.Optional[T] is normalized by parser as typing.Union[None, T],
     and this Union schema type will not have None among its variants. Instead,
@@ -270,7 +279,7 @@ class Union(SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
         super().__init__()
         self.primitive_types = primitive_types
         self.variant_nodes = variant_nodes
-        self.variant_schema_types: t.Set[col.SchemaType] = {
+        self.variant_schema_types: t.Set[meta.SchemaType] = {
             x.typ for _, x in variant_nodes
         }
 
@@ -383,7 +392,7 @@ class Union(SchemaType, metaclass=meta.SubscriptableSchemaTypeM):
 
 
 SUBCLASS_BASED_TO_SCHEMA_TYPE: t.Mapping[
-    t.Tuple[t.Type, ...], t.Type[SchemaType],
+    t.Tuple[t.Type, ...], t.Type[meta.SchemaType],
 ] = {
     (std_enum.Enum,): Enum,
     # Pathlib's PurePath and its derivatives
