@@ -1,12 +1,21 @@
 import re
 import keyword
 import string
-from typing import Any, Type
+from typing import Any, Type, TypeVar, Callable
+
+from colander import TupleSchema, SequenceSchema
+
+from typeit import flags
+from typeit.definitions import OverridesT
+from typeit.schema.nodes import SchemaNode
 
 NORMALIZATION_PREFIX = 'overridden__'
 SUPPORTED_CHARS = string.ascii_letters + string.digits
 
 
+T = TypeVar('T', SchemaNode, TupleSchema, SequenceSchema)
+
+
 def normalize_name(name: str,
                    pattern=re.compile('^([_0-9]+).*$')) -> str:
     """ Some field name patterns are not allowed in NamedTuples
@@ -28,7 +37,7 @@ def is_named_tuple(typ: Type[Any]) -> bool:
     return hasattr(typ, '_fields')
 
 
-def clone_schema_node(node):
+def clone_schema_node(node: T) -> T:
     """ Clonning the node and reassigning the same children,
     because clonning is recursive, but we are only interested
     in a new version of the outermost schema node, the children nodes
@@ -40,3 +49,7 @@ def clone_schema_node(node):
     # the cloned node with new children doesn't affect the original node
     new_node.children = [x for x in node.children]
     return new_node
+
+
+def get_global_name_overrider(overrides: OverridesT) -> Callable[[str], str]:
+    return overrides.get(flags.GlobalNameOverride, flags.Identity)
